@@ -1,10 +1,7 @@
 use crossterm::event::KeyEvent;
 use crossterm::style::Stylize as _;
-use futures::future;
 use ratatui::crossterm::event::{KeyCode, KeyModifiers};
 use std::{
-    collections::HashMap,
-    path::PathBuf,
     sync::{
         atomic::{AtomicBool, AtomicUsize, Ordering},
         Arc,
@@ -12,25 +9,19 @@ use std::{
     time::{Duration, Instant},
 };
 use tokio::{
-    sync::{
-        mpsc::{UnboundedReceiver, UnboundedSender},
-        Semaphore,
-    },
+    sync::mpsc::{UnboundedReceiver, UnboundedSender},
     task::JoinHandle,
 };
 
-use crate::app::{AppEvent, BackgroundProcessingEvent, Event, EventHandlingResult, SearchState};
+use frep_core::search::{FileSearcher, SearchResultWithReplacement};
 
-use scooter_core::replace::replace_in_file;
-use scooter_core::search::SearchResult;
-
-pub use scooter_core::replace::ReplaceResult;
+use crate::app::{AppEvent, BackgroundProcessingEvent, Event, EventHandlingResult};
 
 #[derive(Debug, Eq, PartialEq)]
 pub struct ReplaceState {
     pub num_successes: usize,
     pub num_ignored: usize,
-    pub errors: Vec<SearchResult>,
+    pub errors: Vec<SearchResultWithReplacement>,
     pub replacement_errors_pos: usize,
 }
 
@@ -73,9 +64,6 @@ impl ReplaceState {
 
 #[derive(Debug)]
 pub struct PerformingReplacementState {
-    pub handle: JoinHandle<()>,
-    #[allow(dead_code)]
-    pub processing_sender: UnboundedSender<BackgroundProcessingEvent>,
     pub processing_receiver: UnboundedReceiver<BackgroundProcessingEvent>,
     pub cancelled: Arc<AtomicBool>,
     pub replacement_started: Instant,
@@ -85,16 +73,12 @@ pub struct PerformingReplacementState {
 
 impl PerformingReplacementState {
     pub fn new(
-        handle: JoinHandle<()>,
-        processing_sender: UnboundedSender<BackgroundProcessingEvent>,
         processing_receiver: UnboundedReceiver<BackgroundProcessingEvent>,
         cancelled: Arc<AtomicBool>,
         num_replacements_completed: Arc<AtomicUsize>,
         total_replacements: usize,
     ) -> Self {
         Self {
-            handle,
-            processing_sender,
             processing_receiver,
             cancelled,
             replacement_started: Instant::now(),
@@ -105,70 +89,45 @@ impl PerformingReplacementState {
 }
 
 pub fn perform_replacement(
-    search_state: SearchState,
+    search_results: Vec<SearchResultWithReplacement>,
     background_processing_sender: UnboundedSender<BackgroundProcessingEvent>,
     cancelled: Arc<AtomicBool>,
     replacements_completed: Arc<AtomicUsize>,
     event_sender: UnboundedSender<Event>,
+    validation_search_config: Option<FileSearcher>,
 ) -> JoinHandle<()> {
     tokio::spawn(async move {
         cancelled.store(false, Ordering::Relaxed);
 
-        let (included, num_ignored) = split_results(search_state.results);
-
-        let mut replacements_handle = tokio::spawn(async move {
-            let mut path_groups = HashMap::<PathBuf, Vec<SearchResult>>::new();
-            for res in included {
-                path_groups.entry(res.path.clone()).or_default().push(res);
-            }
-
-            let semaphore = Arc::new(Semaphore::new(8));
-            let mut file_tasks = vec![];
-
-            for (_path, mut results) in path_groups {
-                if cancelled.load(Ordering::Relaxed) {
-                    break;
-                }
-
-                let semaphore = semaphore.clone();
-                let replacements_completed_clone = replacements_completed.clone();
-                let task = tokio::spawn(async move {
-                    let permit = semaphore.acquire_owned().await.unwrap();
-                    if let Err(file_err) = replace_in_file(&mut results) {
-                        for res in &mut results {
-                            res.replace_result = Some(ReplaceResult::Error(file_err.to_string()));
-                        }
-                    }
-                    replacements_completed_clone.fetch_add(results.len(), Ordering::Relaxed);
-
-                    drop(permit);
-                    results
-                });
-                file_tasks.push(task);
-            }
-
-            future::join_all(file_tasks)
-                .await
-                .into_iter()
-                .flat_map(Result::unwrap)
-        });
+        let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
+        let num_ignored = scooter_core::replace::spawn_replace_included(
+            search_results,
+            cancelled,
+            replacements_completed,
+            validation_search_config,
+            move |result| {
+                let _ = tx.send(result); // Ignore error if receiver is dropped
+            },
+        );
 
         let mut rerender_interval = tokio::time::interval(Duration::from_millis(92)); // Slightly random duration so that time taken isn't a round number
 
-        let replacement_results = loop {
+        let mut replacement_results = Vec::new();
+        loop {
             tokio::select! {
-                res = &mut replacements_handle => {
-                    break res.unwrap();
+                res = rx.recv() => match res {
+                    Some(res) => replacement_results.push(res),
+                    None => break,
                 },
                 _ = rerender_interval.tick() => {
                     let _ = event_sender.send(Event::App(AppEvent::Rerender));
                 }
             }
-        };
+        }
 
         let _ = event_sender.send(Event::App(AppEvent::Rerender));
 
-        let stats = calculate_statistics(replacement_results);
+        let stats = frep_core::replace::calculate_statistics(replacement_results);
         // Ignore error: we may have gone back to the previous screen
         let _ = background_processing_sender.send(BackgroundProcessingEvent::ReplacementCompleted(
             ReplaceState {
@@ -181,57 +140,10 @@ pub fn perform_replacement(
     })
 }
 
-#[derive(Clone, Debug, Eq, PartialEq)]
-pub struct ReplaceStats {
-    pub num_successes: usize,
-    pub errors: Vec<SearchResult>,
-}
-
-pub fn calculate_statistics<I>(results: I) -> ReplaceStats
-where
-    I: IntoIterator<Item = SearchResult>,
-{
-    let mut num_successes = 0;
-    let mut errors = vec![];
-
-    results.into_iter().for_each(|res| {
-        assert!(
-            res.included,
-            "Expected only included results, found {res:?}"
-        );
-        match &res.replace_result {
-            Some(ReplaceResult::Success) => {
-                num_successes += 1;
-            }
-            None => {
-                let mut res = res.clone();
-                res.replace_result = Some(ReplaceResult::Error(
-                    "Failed to find search result in file".to_owned(),
-                ));
-                errors.push(res);
-            }
-            Some(ReplaceResult::Error(_)) => {
-                errors.push(res.clone());
-            }
-        }
-    });
-
-    ReplaceStats {
-        num_successes,
-        errors,
-    }
-}
-
-pub fn split_results(results: Vec<SearchResult>) -> (Vec<SearchResult>, usize) {
-    let (included, excluded): (Vec<_>, Vec<_>) = results.into_iter().partition(|res| res.included);
-    let num_ignored = excluded.len();
-    (included, num_ignored)
-}
-
 pub fn format_replacement_results(
     num_successes: usize,
     num_ignored: Option<usize>,
-    errors: Option<&[SearchResult]>,
+    errors: Option<&[SearchResultWithReplacement]>,
 ) -> String {
     let errors_display = if let Some(errors) = errors {
         #[allow(clippy::format_collect)]
@@ -265,24 +177,30 @@ pub fn format_replacement_results(
 mod tests {
     use super::*;
     use crossterm::event::{KeyCode, KeyEvent, KeyModifiers};
-    use scooter_core::line_reader::LineEnding;
+    use frep_core::{
+        line_reader::LineEnding,
+        replace::ReplaceResult,
+        search::{SearchResult, SearchResultWithReplacement},
+    };
     use std::path::PathBuf;
 
-    fn create_search_result(
+    fn create_search_result_with_replacement(
         path: &str,
         line_number: usize,
         line: &str,
         replacement: &str,
         included: bool,
         replace_result: Option<ReplaceResult>,
-    ) -> SearchResult {
-        SearchResult {
-            path: PathBuf::from(path),
-            line_number,
-            line: line.to_string(),
-            line_ending: LineEnding::Lf,
+    ) -> SearchResultWithReplacement {
+        SearchResultWithReplacement {
+            search_result: SearchResult {
+                path: PathBuf::from(path),
+                line_number,
+                line: line.to_string(),
+                line_ending: LineEnding::Lf,
+                included,
+            },
             replacement: replacement.to_string(),
-            included,
             replace_result,
         }
     }
@@ -293,7 +211,7 @@ mod tests {
             num_successes: 5,
             num_ignored: 2,
             errors: vec![
-                create_search_result(
+                create_search_result_with_replacement(
                     "file1.txt",
                     1,
                     "error1",
@@ -301,7 +219,7 @@ mod tests {
                     true,
                     Some(ReplaceResult::Error("err1".to_string())),
                 ),
-                create_search_result(
+                create_search_result_with_replacement(
                     "file2.txt",
                     2,
                     "error2",
@@ -309,7 +227,7 @@ mod tests {
                     true,
                     Some(ReplaceResult::Error("err2".to_string())),
                 ),
-                create_search_result(
+                create_search_result_with_replacement(
                     "file3.txt",
                     3,
                     "error3",
@@ -337,7 +255,7 @@ mod tests {
             num_successes: 5,
             num_ignored: 2,
             errors: vec![
-                create_search_result(
+                create_search_result_with_replacement(
                     "file1.txt",
                     1,
                     "error1",
@@ -345,7 +263,7 @@ mod tests {
                     true,
                     Some(ReplaceResult::Error("err1".to_string())),
                 ),
-                create_search_result(
+                create_search_result_with_replacement(
                     "file2.txt",
                     2,
                     "error2",
@@ -353,7 +271,7 @@ mod tests {
                     true,
                     Some(ReplaceResult::Error("err2".to_string())),
                 ),
-                create_search_result(
+                create_search_result_with_replacement(
                     "file3.txt",
                     3,
                     "error3",
@@ -381,7 +299,7 @@ mod tests {
             num_successes: 5,
             num_ignored: 2,
             errors: vec![
-                create_search_result(
+                create_search_result_with_replacement(
                     "file1.txt",
                     1,
                     "error1",
@@ -389,7 +307,7 @@ mod tests {
                     true,
                     Some(ReplaceResult::Error("err1".to_string())),
                 ),
-                create_search_result(
+                create_search_result_with_replacement(
                     "file2.txt",
                     2,
                     "error2",
@@ -435,38 +353,10 @@ mod tests {
         assert_eq!(result, EventHandlingResult::None);
     }
 
-    #[test]
-    fn test_split_results_all_included() {
-        let result1 = create_search_result("file1.txt", 1, "line1", "repl1", true, None);
-        let result2 = create_search_result("file2.txt", 2, "line2", "repl2", true, None);
-        let result3 = create_search_result("file3.txt", 3, "line3", "repl3", true, None);
-
-        let search_results = vec![result1.clone(), result2.clone(), result3.clone()];
-
-        let (included, num_ignored) = split_results(search_results);
-        assert_eq!(num_ignored, 0);
-        assert_eq!(included, vec![result1, result2, result3]);
-    }
-
-    #[test]
-    fn test_split_results_mixed() {
-        let result1 = create_search_result("file1.txt", 1, "line1", "repl1", true, None);
-        let result2 = create_search_result("file2.txt", 2, "line2", "repl2", false, None);
-        let result3 = create_search_result("file3.txt", 3, "line3", "repl3", true, None);
-        let result4 = create_search_result("file4.txt", 4, "line4", "repl4", false, None);
-
-        let search_results = vec![result1.clone(), result2, result3.clone(), result4];
-
-        let (included, num_ignored) = split_results(search_results);
-        assert_eq!(num_ignored, 2);
-        assert_eq!(included, vec![result1, result3]);
-        assert!(included.iter().all(|r| r.included));
-    }
-
     #[test]
     fn test_calculate_statistics_all_success() {
         let results = vec![
-            create_search_result(
+            create_search_result_with_replacement(
                 "file1.txt",
                 1,
                 "line1",
@@ -474,7 +364,7 @@ mod tests {
                 true,
                 Some(ReplaceResult::Success),
             ),
-            create_search_result(
+            create_search_result_with_replacement(
                 "file2.txt",
                 2,
                 "line2",
@@ -482,7 +372,7 @@ mod tests {
                 true,
                 Some(ReplaceResult::Success),
             ),
-            create_search_result(
+            create_search_result_with_replacement(
                 "file3.txt",
                 3,
                 "line3",
@@ -492,14 +382,14 @@ mod tests {
             ),
         ];
 
-        let stats = calculate_statistics(results);
+        let stats = frep_core::replace::calculate_statistics(results);
         assert_eq!(stats.num_successes, 3);
         assert_eq!(stats.errors.len(), 0);
     }
 
     #[test]
     fn test_calculate_statistics_with_errors() {
-        let error_result = create_search_result(
+        let error_result = create_search_result_with_replacement(
             "file2.txt",
             2,
             "line2",
@@ -508,7 +398,7 @@ mod tests {
             Some(ReplaceResult::Error("test error".to_string())),
         );
         let results = vec![
-            create_search_result(
+            create_search_result_with_replacement(
                 "file1.txt",
                 1,
                 "line1",
@@ -517,7 +407,7 @@ mod tests {
                 Some(ReplaceResult::Success),
             ),
             error_result.clone(),
-            create_search_result(
+            create_search_result_with_replacement(
                 "file3.txt",
                 3,
                 "line3",
@@ -527,16 +417,19 @@ mod tests {
             ),
         ];
 
-        let stats = calculate_statistics(results);
+        let stats = frep_core::replace::calculate_statistics(results);
         assert_eq!(stats.num_successes, 2);
         assert_eq!(stats.errors.len(), 1);
-        assert_eq!(stats.errors[0].path, error_result.path);
+        assert_eq!(
+            stats.errors[0].search_result.path,
+            error_result.search_result.path
+        );
     }
 
     #[test]
     fn test_calculate_statistics_with_none_results() {
         let results = vec![
-            create_search_result(
+            create_search_result_with_replacement(
                 "file1.txt",
                 1,
                 "line1",
@@ -544,8 +437,8 @@ mod tests {
                 true,
                 Some(ReplaceResult::Success),
             ),
-            create_search_result("file2.txt", 2, "line2", "repl2", true, None), // This should be treated as an error
-            create_search_result(
+            create_search_result_with_replacement("file2.txt", 2, "line2", "repl2", true, None), // This should be treated as an error
+            create_search_result_with_replacement(
                 "file3.txt",
                 3,
                 "line3",
@@ -555,10 +448,13 @@ mod tests {
             ),
         ];
 
-        let stats = calculate_statistics(results);
+        let stats = frep_core::replace::calculate_statistics(results);
         assert_eq!(stats.num_successes, 2);
         assert_eq!(stats.errors.len(), 1);
-        assert_eq!(stats.errors[0].path, PathBuf::from("file2.txt"));
+        assert_eq!(
+            stats.errors[0].search_result.path,
+            PathBuf::from("file2.txt")
+        );
         assert_eq!(
             stats.errors[0].replace_result,
             Some(ReplaceResult::Error(
@@ -578,7 +474,7 @@ mod tests {
 
     #[test]
     fn test_format_replacement_results_with_errors() {
-        let error_result = create_search_result(
+        let error_result = create_search_result_with_replacement(
             "file.txt",
             10,
             "line",
