@@ -1,4 +1,4 @@
-use anyhow::{bail, Context};
+use anyhow::bail;
 use itertools::Itertools;
 use lru::LruCache;
 use ratatui::{
@@ -8,7 +8,10 @@ use ratatui::{
     widgets::{Block, Cell, Clear, List, ListItem, Padding, Paragraph, Row, Table},
     Frame,
 };
-use similar::{Change, ChangeTag, TextDiff};
+use scooter_core::{
+    diff::{line_diff, Diff, DiffColour},
+    utils::{relative_path_from, split_indexed_lines, strip_control_chars},
+};
 use std::{
     cmp::min,
     fs, iter,
@@ -24,16 +27,15 @@ use syntect::{
 use tokio::sync::mpsc::UnboundedSender;
 
 use crate::{
-    app::{App, AppError, AppEvent, Event, Popup, Screen, SearchState},
-    fields::{Field, SearchField, NUM_SEARCH_FIELDS},
+    app::{App, AppError, AppEvent, Event, FocussedSection, Popup, Screen, SearchState},
+    config::Config,
+    fields::{Field, SearchField, SearchFields, NUM_SEARCH_FIELDS},
     replace::{PerformingReplacementState, ReplaceState},
-    utils::{
-        group_by, largest_range_centered_on, last_n_chars, read_lines_range,
-        read_lines_range_highlighted, relative_path_from, strip_control_chars, HighlightedLine,
-    },
+    utils::{last_n_chars, read_lines_range_highlighted, HighlightedLine},
 };
 
-use scooter_core::search::SearchResult;
+use frep_core::search::SearchResultWithReplacement;
+use scooter_core::utils::read_lines_range;
 
 use super::colour::to_ratatui_colour;
 
@@ -111,34 +113,44 @@ impl SearchField {
     }
 }
 
-fn render_search_view(frame: &mut Frame<'_>, app: &mut App, area: Rect) {
-    let area = default_width(area);
-    let areas: [Rect; NUM_SEARCH_FIELDS] = Layout::vertical(iter::repeat_n(
-        Constraint::Length(3),
-        app.search_fields.fields.len(),
+static SEARCH_FIELD_HEIGHT: u16 = 3;
+static NUM_SEARCH_FIELDS_TRUNCATED: u16 = 2;
+
+fn render_search_fields(
+    frame: &mut Frame<'_>,
+    search_fields: &SearchFields,
+    config: &Config,
+    show_popup: bool,
+    num_search_fields_to_render: u16,
+    is_focussed: bool,
+    area: Rect,
+) {
+    let areas = Layout::vertical(iter::repeat_n(
+        Constraint::Length(SEARCH_FIELD_HEIGHT),
+        num_search_fields_to_render as usize,
     ))
     .flex(Flex::Center)
-    .areas(area);
+    .split(area);
 
-    app.search_fields
+    search_fields
         .fields
         .iter()
-        .zip(areas)
+        .zip(areas.iter())
         .enumerate()
-        .for_each(|(idx, (search_field, field_area))| {
+        .for_each(|(idx, (search_field, &field_area))| {
             search_field.render(
                 frame,
                 field_area,
-                idx == app.search_fields.highlighted,
-                app.config.search.disable_prepopulated_fields,
+                is_focussed && idx == search_fields.highlighted,
+                config.search.disable_prepopulated_fields,
             );
         });
 
-    if !app.show_popup() {
-        let field = app.search_fields.highlighted_field();
-        if !field.set_by_cli || !app.config.search.disable_prepopulated_fields {
+    if is_focussed && !show_popup {
+        let field = search_fields.highlighted_field();
+        if !field.set_by_cli || !config.search.disable_prepopulated_fields {
             if let Some(cursor_pos) = field.cursor_pos() {
-                let highlighted_area = areas[app.search_fields.highlighted];
+                let highlighted_area = areas[search_fields.highlighted];
 
                 frame.set_cursor_position(Position {
                     x: highlighted_area.x + u16::try_from(cursor_pos).unwrap_or(0) + 1,
@@ -150,7 +162,8 @@ fn render_search_view(frame: &mut Frame<'_>, app: &mut App, area: Rect) {
 }
 
 fn default_width(area: Rect) -> Rect {
-    width(area, 80)
+    let width_percentage = if area.width >= 300 { 80 } else { 90 };
+    width(area, width_percentage)
 }
 
 fn width(area: Rect, percentage: u16) -> Rect {
@@ -160,77 +173,25 @@ fn width(area: Rect, percentage: u16) -> Rect {
     area
 }
 
-#[derive(Debug, PartialEq, Eq, Clone)]
-pub struct Diff {
-    pub text: String,
-    pub fg_colour: Color,
-    pub bg_colour: Option<Color>,
+fn diff_col_to_ratatui(colour: &DiffColour) -> Color {
+    match colour {
+        DiffColour::Red => Color::Red,
+        DiffColour::Green => Color::Green,
+        DiffColour::Black => Color::Black,
+    }
 }
 
 fn diff_to_line(diff: Vec<&Diff>) -> Line<'static> {
     let diff_iter = diff.into_iter().map(|d| {
-        let mut style = Style::new().fg(d.fg_colour);
-        if let Some(bg) = d.bg_colour {
-            style = style.bg(bg);
+        let mut style = Style::new().fg(diff_col_to_ratatui(&d.fg_colour));
+        if let Some(bg) = &d.bg_colour {
+            style = style.bg(diff_col_to_ratatui(bg));
         }
         Span::styled(strip_control_chars(&d.text), style)
     });
     diff_iter.collect()
 }
 
-pub fn line_diff<'a>(old_line: &'a str, new_line: &'a str) -> (Vec<Diff>, Vec<Diff>) {
-    let diff = TextDiff::configure()
-        .algorithm(similar::Algorithm::Myers)
-        .timeout(std::time::Duration::from_millis(100))
-        .diff_chars(old_line, new_line);
-
-    let mut old_spans = vec![Diff {
-        text: "- ".to_owned(),
-        fg_colour: Color::Red,
-        bg_colour: None,
-    }];
-    let mut new_spans = vec![Diff {
-        text: "+ ".to_owned(),
-        fg_colour: Color::Green,
-        bg_colour: None,
-    }];
-
-    for change_group in group_by(diff.iter_all_changes(), |c1, c2| c1.tag() == c2.tag()) {
-        let first_change = change_group.first().unwrap(); // group_by should never return an empty group
-        let text = change_group.iter().map(Change::value).collect();
-        match first_change.tag() {
-            ChangeTag::Delete => {
-                old_spans.push(Diff {
-                    text,
-                    fg_colour: Color::Black,
-                    bg_colour: Some(Color::Red),
-                });
-            }
-            ChangeTag::Insert => {
-                new_spans.push(Diff {
-                    text,
-                    fg_colour: Color::Black,
-                    bg_colour: Some(Color::Green),
-                });
-            }
-            ChangeTag::Equal => {
-                old_spans.push(Diff {
-                    text: text.clone(),
-                    fg_colour: Color::Red,
-                    bg_colour: None,
-                });
-                new_spans.push(Diff {
-                    text,
-                    fg_colour: Color::Green,
-                    bg_colour: None,
-                });
-            }
-        }
-    }
-
-    (old_spans, new_spans)
-}
-
 fn display_duration(duration: Duration) -> String {
     let seconds = duration.as_secs();
     let milliseconds = duration.subsec_millis();
@@ -238,7 +199,7 @@ fn display_duration(duration: Duration) -> String {
 }
 
 #[allow(clippy::too_many_arguments, clippy::too_many_lines)]
-fn render_confirmation_view(
+fn render_search_results(
     frame: &mut Frame<'_>,
     is_complete: bool,
     search_state: &mut SearchState,
@@ -248,9 +209,10 @@ fn render_confirmation_view(
     theme: Option<&Theme>,
     true_colour: bool,
     event_sender: UnboundedSender<Event>,
+    area_is_focussed: bool,
+    preview_update_status: Option<(usize, usize)>,
 ) {
-    let split_view = area.width >= 110;
-    let area = width(area, 90);
+    let small_screen = area.width <= 110;
 
     let [num_results_area, results_area, _] = Layout::vertical([
         Constraint::Length(2),
@@ -268,12 +230,13 @@ fn render_confirmation_view(
         num_results,
         is_complete,
         time_taken,
+        preview_update_status,
     );
 
-    let num_to_render = if split_view {
-        results_area.height as usize
-    } else {
+    let num_to_render = if small_screen {
         5
+    } else {
+        results_area.height as usize
     };
 
     search_state.num_displayed = Some(num_to_render);
@@ -290,15 +253,7 @@ fn render_confirmation_view(
         );
     }
 
-    let (list_area, preview_area) = if split_view {
-        let [list_area, _, preview_area] = Layout::horizontal([
-            Constraint::Fill(2),
-            Constraint::Length(1),
-            Constraint::Fill(3),
-        ])
-        .areas(results_area);
-        (list_area, preview_area)
-    } else {
+    let (list_area, preview_area) = if small_screen {
         let [list_area, _, preview_area] = Layout::vertical([
             #[allow(clippy::cast_possible_truncation)]
             Constraint::Length(num_to_render as u16),
@@ -307,10 +262,23 @@ fn render_confirmation_view(
         ])
         .areas(results_area);
         (list_area, preview_area)
+    } else {
+        let [list_area, _, preview_area] = Layout::horizontal([
+            Constraint::Fill(2),
+            Constraint::Length(1),
+            Constraint::Fill(3),
+        ])
+        .areas(results_area);
+        (list_area, preview_area)
     };
 
-    let search_results =
-        build_search_results(search_state, base_path, list_area.width, num_to_render);
+    let search_results = build_search_results(
+        search_state,
+        base_path,
+        list_area.width,
+        num_to_render,
+        area_is_focussed,
+    );
     let search_results_list = search_results
         .iter()
         .map(|SearchResultLines { file_path, .. }| ListItem::new(file_path.clone()));
@@ -343,6 +311,7 @@ fn render_num_results(
     num_results: usize,
     is_complete: bool,
     time_taken: Duration,
+    num_replacements_updates_in_progress: Option<(usize, usize)>,
 ) {
     let left_content_1 = format!("Results: {num_results}");
     let left_content_2 = if is_complete {
@@ -350,11 +319,12 @@ fn render_num_results(
     } else {
         " [Still searching...]"
     };
+    let mid_content = preview_update_status(num_replacements_updates_in_progress);
     let right_content = format!(" [Time taken: {}]", display_duration(time_taken));
-    let spacers = " ".repeat(
-        (area.width as usize)
-            .saturating_sub(left_content_1.len() + left_content_2.len() + right_content.len()),
+    let num_total_spacers = (area.width as usize).saturating_sub(
+        left_content_1.len() + left_content_2.len() + mid_content.len() + right_content.len(),
     );
+    let spacers_each_side = " ".repeat(num_total_spacers / 2);
 
     let accessory_colour = if is_complete {
         Color::Green
@@ -366,18 +336,36 @@ fn render_num_results(
         Line::from(vec![
             Span::raw(left_content_1),
             Span::raw(left_content_2).fg(accessory_colour),
-            Span::raw(spacers),
+            Span::raw(spacers_each_side.clone()),
+            Span::raw(mid_content).fg(Color::Blue),
+            Span::raw(spacers_each_side),
             Span::raw(right_content).fg(accessory_colour),
         ]),
         area,
     );
 }
 
+fn preview_update_status(num_replacements_updates_in_progress: Option<(usize, usize)>) -> String {
+    if let Some((complete, total)) = num_replacements_updates_in_progress {
+        // Avoid flickering - only show if it will take some time
+        if total >= 10_000 {
+            #[allow(clippy::cast_precision_loss)]
+            return format!(
+                "[Updating preview: {complete}/{total} ({perc:.2}%)]",
+                perc = ((complete as f64) / (total as f64)) * 100.0
+            );
+        }
+    }
+
+    String::new()
+}
+
 fn build_search_results<'a>(
     search_state: &'a mut SearchState,
     base_path: &Path,
     width: u16,
     num_to_render: usize,
+    area_is_focussed: bool,
 ) -> Vec<SearchResultLines<'a>> {
     search_state
         .results
@@ -393,6 +381,7 @@ fn build_search_results<'a>(
                 result,
                 base_path,
                 width,
+                area_is_focussed,
             )
         })
         .collect()
@@ -451,7 +440,6 @@ pub(crate) fn highlighted_lines_cache() -> &'static HighlightedLinesCache {
 }
 
 fn spawn_highlight_full_file(path: PathBuf, theme: Theme, event_sender: UnboundedSender<Event>) {
-    // TODO: cancel thread if app closes
     tokio::spawn(async move {
         match fs::metadata(&path) {
             Ok(metadata) => {
@@ -527,24 +515,25 @@ fn build_preview_list<'a>(
     true_colour: bool,
     event_sender: UnboundedSender<Event>,
 ) -> anyhow::Result<List<'a>> {
-    let line_idx = selected.search_result.line_number - 1;
+    let line_idx = selected.result.search_result.line_number - 1;
     let start = line_idx.saturating_sub(num_lines_to_show);
     let end = line_idx + num_lines_to_show;
 
     if let Some(theme) = syntax_highlighting_theme {
         let lines = read_lines_range_highlighted_with_cache(
-            &selected.search_result.path,
+            &selected.result.search_result.path,
             start,
             end,
             theme,
             event_sender,
         )?;
         // `num_lines_to_show - 1` because diff takes up 2 lines
-        let Ok((before, cur, after)) = split_indexed_lines(lines, line_idx, num_lines_to_show - 1)
+        let Ok((before, cur, after)) =
+            split_indexed_lines(lines, line_idx, num_lines_to_show.saturating_sub(1))
         else {
             bail!("File has changed since search (lines have changed)");
         };
-        if *cur.1.iter().map(|(_, s)| s).join("") != selected.search_result.line {
+        if *cur.1.iter().map(|(_, s)| s).join("") != selected.result.search_result.line {
             bail!("File has changed since search (lines don't match)");
         }
 
@@ -568,10 +557,12 @@ fn build_preview_list<'a>(
             Ok(list)
         }
     } else {
-        let lines = read_lines_range(&selected.search_result.path, start, end)?;
+        let lines = read_lines_range(&selected.result.search_result.path, start, end)?;
         let (before, cur, after) =
             split_indexed_lines(lines.collect::<Vec<_>>(), line_idx, num_lines_to_show - 1)?; // -1 because diff takes up 2 lines
-        assert_eq!(*cur.1, selected.search_result.line);
+        if *cur.1 != selected.result.search_result.line {
+            bail!("File has changed since search (lines don't match)");
+        }
 
         Ok(List::new(
             before
@@ -586,54 +577,24 @@ fn build_preview_list<'a>(
     }
 }
 
-#[allow(clippy::type_complexity)]
-fn split_indexed_lines<T>(
-    indexed_lines: Vec<(usize, T)>,
-    line_idx: usize,
-    num_lines_to_show: usize,
-) -> anyhow::Result<(Vec<(usize, T)>, (usize, T), Vec<(usize, T)>)> {
-    let file_start = indexed_lines.first().context("No lines found")?.0;
-    let file_end = indexed_lines.last().context("No lines found")?.0;
-    let (new_start, new_end) = largest_range_centered_on(
-        line_idx,
-        file_start,
-        file_end,
-        NonZeroUsize::new(num_lines_to_show).context("preview will have height 0")?,
-    )?;
-
-    let mut filtered_lines = indexed_lines
-        .into_iter()
-        .skip_while(|(idx, _)| *idx < new_start)
-        .take_while(|(idx, _)| *idx <= new_end)
-        .collect::<Vec<_>>();
-
-    let position = filtered_lines
-        .iter()
-        .position(|(idx, _)| *idx == line_idx)
-        .context("Couldn't find line in file")?;
-    let after = filtered_lines.split_off(position + 1);
-    let current = filtered_lines.pop().unwrap();
-
-    Ok((filtered_lines, current, after))
-}
-
 struct SearchResultLines<'a> {
     file_path: Line<'a>,
     old_line_diff: Line<'static>,
     new_line_diff: Line<'static>,
     is_primary_selected: bool,
-    search_result: &'a SearchResult,
+    result: &'a SearchResultWithReplacement,
 }
 
 fn search_result<'a>(
     idx: usize,
     is_selected: bool,
     is_primary_selected: bool,
-    result: &'a SearchResult,
+    result: &'a SearchResultWithReplacement,
     base_path: &Path,
     list_area_width: u16,
+    area_is_focussed: bool,
 ) -> SearchResultLines<'a> {
-    let (old_line, new_line) = line_diff(&result.line, &result.replacement);
+    let (old_line, new_line) = line_diff(&result.search_result.line, &result.replacement);
     let old_line = old_line
         .iter()
         .take(list_area_width as usize)
@@ -651,11 +612,12 @@ fn search_result<'a>(
             is_selected,
             is_primary_selected,
             list_area_width,
+            area_is_focussed,
         ),
         old_line_diff: diff_to_line(old_line),
         new_line_diff: diff_to_line(new_line),
         is_primary_selected,
-        search_result: result,
+        result,
     }
 }
 
@@ -663,16 +625,17 @@ static TRUNCATION_PREFIX: &str = "…";
 
 fn file_path_line<'a>(
     idx: usize,
-    result: &SearchResult,
+    result: &SearchResultWithReplacement,
     base_path: &Path,
     is_selected: bool,
     is_primary_selected: bool,
     list_area_width: u16,
+    area_is_focussed: bool,
 ) -> Line<'a> {
     let mut file_path_style = Style::new();
-    if is_selected {
+    if area_is_focussed && is_selected {
         file_path_style = file_path_style
-            .bg(match (result.included, is_primary_selected) {
+            .bg(match (result.search_result.included, is_primary_selected) {
                 (true, true) => Color::Blue,
                 (true, false) => Color::Indexed(26),
                 (false, true) => Color::Red,
@@ -682,38 +645,40 @@ fn file_path_line<'a>(
     }
 
     let right_content = format!(" ({})", idx + 1);
-    let right_content_len = right_content.len();
-    let left_content = format!("[{}] ", if result.included { 'x' } else { ' ' },);
-    let left_content_len = left_content.chars().count();
-    let centre_content = format!(
-        "{}:{}",
-        relative_path_from(base_path, &result.path),
-        result.line_number,
+    let right_content_len = right_content.chars().count();
+    let left_content = format!(
+        "[{}] ",
+        if result.search_result.included {
+            'x'
+        } else {
+            ' '
+        },
     );
-    let centre_content_space =
-        (list_area_width as usize).saturating_sub(left_content_len + right_content_len);
-    let centre_content = if centre_content.len() > centre_content_space {
-        let truncated = last_n_chars(
-            &centre_content,
-            centre_content_space - TRUNCATION_PREFIX.chars().count(),
-        );
-        format!("{TRUNCATION_PREFIX}{truncated}").to_string()
-    } else {
-        centre_content
-    };
+    let left_content_len = left_content.chars().count();
+    let mut path = relative_path_from(base_path, &result.search_result.path);
+    let line_num = format!(":{}", result.search_result.line_number);
+    let line_num_len = line_num.chars().count();
+    let path_space = (list_area_width as usize)
+        .saturating_sub(left_content_len + line_num_len + right_content_len);
+    if path.len() > path_space {
+        let truncated = last_n_chars(&path, path_space - TRUNCATION_PREFIX.chars().count());
+        path = format!("{TRUNCATION_PREFIX}{truncated}").to_string();
+    }
+    let path_len = path.chars().count();
     let spacers = " ".repeat(
         (list_area_width as usize)
-            .saturating_sub(left_content_len + centre_content.len() + right_content_len),
+            .saturating_sub(left_content_len + path_len + line_num_len + right_content_len),
     );
 
-    let accessory_colour = if is_selected {
+    let accessory_colour = if area_is_focussed && is_selected {
         Color::Indexed(255)
     } else {
         Color::Blue
     };
     Line::from(vec![
         Span::raw(left_content).style(accessory_colour),
-        Span::raw(centre_content),
+        Span::raw(path),
+        Span::raw(line_num).style(accessory_colour),
         Span::raw(spacers),
         Span::raw(right_content).style(accessory_colour),
     ])
@@ -766,7 +731,7 @@ fn render_results_errors(area: Rect, replace_state: &ReplaceState, frame: &mut F
         .errors
         .iter()
         .skip(replace_state.replacement_errors_pos)
-        .take(list_area.height as usize / 3 + 1) // TODO: don't hardcode height
+        .take(list_area.height as usize / 3 + 1)
         .map(error_result);
 
     render_results_tallies(results_area, frame, replace_state);
@@ -853,7 +818,7 @@ fn render_performing_replacement_view(
     );
 }
 
-fn error_result(result: &SearchResult) -> [ratatui::widgets::ListItem<'static>; 3] {
+fn error_result(result: &SearchResultWithReplacement) -> [ratatui::widgets::ListItem<'static>; 3] {
     let (path_display, error) = result.display_error();
 
     [
@@ -886,33 +851,53 @@ pub fn render(app: &mut App, frame: &mut Frame<'_>) {
     render_key_hints(app, frame, footer_area);
 
     let base_path = &app.directory;
+    let show_popup = app.show_popup();
     match &mut app.current_screen {
-        Screen::SearchFields => render_search_view(frame, app, content_area),
-        Screen::SearchProgressing(ref mut s) => {
-            render_confirmation_view(
+        Screen::SearchFields(ref mut search_fields_state) => {
+            let num_search_fields_to_render = match search_fields_state.focussed_section {
+                FocussedSection::SearchFields => NUM_SEARCH_FIELDS,
+                FocussedSection::SearchResults => NUM_SEARCH_FIELDS_TRUNCATED,
+            };
+            let area = default_width(content_area);
+            let [fields, _, results] = Layout::vertical([
+                Constraint::Length(num_search_fields_to_render * SEARCH_FIELD_HEIGHT),
+                Constraint::Length(1),
+                Constraint::Fill(1),
+            ])
+            .flex(Flex::Center)
+            .areas(area);
+
+            render_search_fields(
                 frame,
-                false,
-                &mut s.search_state,
-                s.search_started.elapsed(),
-                base_path,
-                content_area,
-                app.config.get_theme(),
-                app.config.style.true_color,
-                app.event_sender.clone(),
-            );
-        }
-        Screen::SearchComplete(ref mut state) => {
-            render_confirmation_view(
-                frame,
-                true,
-                &mut state.search_state,
-                state.search_time_taken,
-                base_path,
-                content_area,
-                app.config.get_theme(),
-                app.config.style.true_color,
-                app.event_sender.clone(),
+                &app.search_fields,
+                &app.config,
+                show_popup,
+                num_search_fields_to_render,
+                search_fields_state.focussed_section == FocussedSection::SearchFields,
+                fields,
             );
+
+            let replacements_in_progress = search_fields_state.replacements_in_progress();
+            if let Some(ref mut state) = &mut search_fields_state.search_state {
+                let (is_complete, elapsed) = if let Some(completed) = state.search_completed {
+                    (true, completed.duration_since(state.search_started))
+                } else {
+                    (false, state.search_started.elapsed())
+                };
+                render_search_results(
+                    frame,
+                    is_complete,
+                    state,
+                    elapsed,
+                    base_path,
+                    results,
+                    app.config.get_theme(),
+                    app.config.style.true_color,
+                    app.event_sender.clone(),
+                    search_fields_state.focussed_section == FocussedSection::SearchResults,
+                    replacements_in_progress,
+                );
+            }
         }
         Screen::PerformingReplacement(state) => {
             render_performing_replacement_view(frame, content_area, state);
@@ -1047,230 +1032,8 @@ fn create_popup_block(title: &str) -> Block<'_> {
 
 #[cfg(test)]
 mod tests {
-    use ratatui::style::Color;
-
     use super::*;
 
-    #[test]
-    fn test_identical_lines() {
-        let (old_actual, new_actual) = line_diff("hello", "hello");
-
-        let old_expected = vec![
-            Diff {
-                text: "- ".to_owned(),
-                fg_colour: Color::Red,
-                bg_colour: None,
-            },
-            Diff {
-                text: "hello".to_owned(),
-                fg_colour: Color::Red,
-                bg_colour: None,
-            },
-        ];
-
-        let new_expected = vec![
-            Diff {
-                text: "+ ".to_owned(),
-                fg_colour: Color::Green,
-                bg_colour: None,
-            },
-            Diff {
-                text: "hello".to_owned(),
-                fg_colour: Color::Green,
-                bg_colour: None,
-            },
-        ];
-
-        assert_eq!(old_expected, old_actual);
-        assert_eq!(new_expected, new_actual);
-    }
-
-    #[test]
-    fn test_single_char_difference() {
-        let (old_actual, new_actual) = line_diff("hello", "hallo");
-
-        let old_expected = vec![
-            Diff {
-                text: "- ".to_owned(),
-                fg_colour: Color::Red,
-                bg_colour: None,
-            },
-            Diff {
-                text: "h".to_owned(),
-                fg_colour: Color::Red,
-                bg_colour: None,
-            },
-            Diff {
-                text: "e".to_owned(),
-                fg_colour: Color::Black,
-                bg_colour: Some(Color::Red),
-            },
-            Diff {
-                text: "llo".to_owned(),
-                fg_colour: Color::Red,
-                bg_colour: None,
-            },
-        ];
-
-        let new_expected = vec![
-            Diff {
-                text: "+ ".to_owned(),
-                fg_colour: Color::Green,
-                bg_colour: None,
-            },
-            Diff {
-                text: "h".to_owned(),
-                fg_colour: Color::Green,
-                bg_colour: None,
-            },
-            Diff {
-                text: "a".to_owned(),
-                fg_colour: Color::Black,
-                bg_colour: Some(Color::Green),
-            },
-            Diff {
-                text: "llo".to_owned(),
-                fg_colour: Color::Green,
-                bg_colour: None,
-            },
-        ];
-
-        assert_eq!(old_expected, old_actual);
-        assert_eq!(new_expected, new_actual);
-    }
-
-    #[test]
-    fn test_completely_different_strings() {
-        let (old_actual, new_actual) = line_diff("foo", "bar");
-
-        let old_expected = vec![
-            Diff {
-                text: "- ".to_owned(),
-                fg_colour: Color::Red,
-                bg_colour: None,
-            },
-            Diff {
-                text: "foo".to_owned(),
-                fg_colour: Color::Black,
-                bg_colour: Some(Color::Red),
-            },
-        ];
-
-        let new_expected = vec![
-            Diff {
-                text: "+ ".to_owned(),
-                fg_colour: Color::Green,
-                bg_colour: None,
-            },
-            Diff {
-                text: "bar".to_owned(),
-                fg_colour: Color::Black,
-                bg_colour: Some(Color::Green),
-            },
-        ];
-
-        assert_eq!(old_expected, old_actual);
-        assert_eq!(new_expected, new_actual);
-    }
-
-    #[test]
-    fn test_empty_strings() {
-        let (old_actual, new_actual) = line_diff("", "");
-
-        let old_expected = vec![Diff {
-            text: "- ".to_owned(),
-            fg_colour: Color::Red,
-            bg_colour: None,
-        }];
-
-        let new_expected = vec![Diff {
-            text: "+ ".to_owned(),
-            fg_colour: Color::Green,
-            bg_colour: None,
-        }];
-
-        assert_eq!(old_expected, old_actual);
-        assert_eq!(new_expected, new_actual);
-    }
-
-    #[test]
-    fn test_addition_at_end() {
-        let (old_actual, new_actual) = line_diff("hello", "hello!");
-
-        let old_expected = vec![
-            Diff {
-                text: "- ".to_owned(),
-                fg_colour: Color::Red,
-                bg_colour: None,
-            },
-            Diff {
-                text: "hello".to_owned(),
-                fg_colour: Color::Red,
-                bg_colour: None,
-            },
-        ];
-
-        let new_expected = vec![
-            Diff {
-                text: "+ ".to_owned(),
-                fg_colour: Color::Green,
-                bg_colour: None,
-            },
-            Diff {
-                text: "hello".to_owned(),
-                fg_colour: Color::Green,
-                bg_colour: None,
-            },
-            Diff {
-                text: "!".to_owned(),
-                fg_colour: Color::Black,
-                bg_colour: Some(Color::Green),
-            },
-        ];
-
-        assert_eq!(old_expected, old_actual);
-        assert_eq!(new_expected, new_actual);
-    }
-
-    #[test]
-    fn test_addition_at_start() {
-        let (old_actual, new_actual) = line_diff("hello", "!hello");
-
-        let old_expected = vec![
-            Diff {
-                text: "- ".to_owned(),
-                fg_colour: Color::Red,
-                bg_colour: None,
-            },
-            Diff {
-                text: "hello".to_owned(),
-                fg_colour: Color::Red,
-                bg_colour: None,
-            },
-        ];
-
-        let new_expected = vec![
-            Diff {
-                text: "+ ".to_owned(),
-                fg_colour: Color::Green,
-                bg_colour: None,
-            },
-            Diff {
-                text: "!".to_owned(),
-                fg_colour: Color::Black,
-                bg_colour: Some(Color::Green),
-            },
-            Diff {
-                text: "hello".to_owned(),
-                fg_colour: Color::Green,
-                bg_colour: None,
-            },
-        ];
-
-        assert_eq!(old_expected, old_actual);
-        assert_eq!(new_expected, new_actual);
-    }
-
     #[test]
     fn test_split_lines_centered() {
         let lines: Vec<(usize, String)> =
