@@ -1,6 +1,7 @@
+use frep_core::validation::SearchConfiguration;
 use scooter::{
     headless::run_headless, test_with_both_regex_modes,
-    test_with_both_regex_modes_and_fixed_strings, validation::SearchConfiguration,
+    test_with_both_regex_modes_and_fixed_strings,
 };
 
 mod utils;
@@ -26,11 +27,11 @@ test_with_both_regex_modes_and_fixed_strings!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: "TEST_PATTERN".to_string(),
-            replacement_text: "REPLACEMENT".to_string(),
+            search_text: "TEST_PATTERN",
+            replacement_text: "REPLACEMENT",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -40,10 +41,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 4".to_owned(),
-        );
+        assert_eq!(result.unwrap(), "Success: 3 files updated".to_owned(),);
 
         assert_test_files!(
             &temp_dir,
@@ -87,11 +85,11 @@ test_with_both_regex_modes!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: r"\d{3}".to_string(),
-            replacement_text: "XXX".to_string(),
+            search_text: r"\d{3}",
+            replacement_text: "XXX",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings: false,
             match_case: true,
@@ -101,10 +99,7 @@ test_with_both_regex_modes!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 6".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 3 files updated".to_string(),);
 
         assert_test_files!(
             &temp_dir,
@@ -142,11 +137,11 @@ test_with_both_regex_modes!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: r"username: (\w+), email: ([^@]+)@".to_string(),
-            replacement_text: "user: $1 (contact: $2 at".to_string(),
+            search_text: r"username: (\w+), email: ([^@]+)@",
+            replacement_text: "user: $1 (contact: $2 at",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings: false,
             match_case: true,
@@ -156,17 +151,14 @@ test_with_both_regex_modes!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 2".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 1 file updated".to_string(),);
 
         let search_config = SearchConfiguration {
-            search_text: r"\[(\d{4})-(\d{2})-(\d{2})\]".to_string(),
-            replacement_text: "[$3/$2/$1]".to_string(),
+            search_text: r"\[(\d{4})-(\d{2})-(\d{2})\]",
+            replacement_text: "[$3/$2/$1]",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "logs.txt".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some("logs.txt"),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings: false,
             match_case: true,
@@ -176,10 +168,7 @@ test_with_both_regex_modes!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 2".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 1 file updated".to_string(),);
 
         assert_test_files!(
             &temp_dir,
@@ -221,11 +210,11 @@ async fn test_headless_advanced_regex_features() -> anyhow::Result<()> {
 
     // Negative lookahead - match 'let' but not 'let mut'
     let search_config = SearchConfiguration {
-        search_text: r"let(?!\s+mut)".to_string(),
-        replacement_text: "const".to_string(),
+        search_text: r"let(?!\s+mut)",
+        replacement_text: "const",
         directory: temp_dir.path().to_path_buf(),
-        include_globs: "code.rs".to_string(),
-        exclude_globs: "".to_string(),
+        include_globs: Some("code.rs"),
+        exclude_globs: Some(""),
         include_hidden: false,
         fixed_strings: false,
         match_case: true,
@@ -234,18 +223,15 @@ async fn test_headless_advanced_regex_features() -> anyhow::Result<()> {
     };
 
     let result = run_headless(search_config);
-    assert_eq!(
-        result.unwrap(),
-        "Successful replacements (lines): 1".to_string(),
-    );
+    assert_eq!(result.unwrap(), "Success: 1 file updated".to_string(),);
 
     // Positive lookbehind - match numbers after headings
     let search_config = SearchConfiguration {
-        search_text: r"(?<=# )[A-Za-z]+\s+(\d+)".to_string(),
-        replacement_text: "Section $1".to_string(),
+        search_text: r"(?<=# )[A-Za-z]+\s+(\d+)",
+        replacement_text: "Section $1",
         directory: temp_dir.path().to_path_buf(),
-        include_globs: "*.md".to_string(),
-        exclude_globs: "".to_string(),
+        include_globs: Some("*.md"),
+        exclude_globs: Some(""),
         include_hidden: false,
         fixed_strings: false,
         match_case: true,
@@ -254,18 +240,15 @@ async fn test_headless_advanced_regex_features() -> anyhow::Result<()> {
     };
 
     let result = run_headless(search_config);
-    assert_eq!(
-        result.unwrap(),
-        "Successful replacements (lines): 1".to_string(),
-    );
+    assert_eq!(result.unwrap(), "Success: 1 file updated".to_string(),);
 
     // Add spaces after commas in CSV file
     let search_config = SearchConfiguration {
-        search_text: ",".to_string(),
-        replacement_text: ", ".to_string(),
+        search_text: ",",
+        replacement_text: ", ",
         directory: temp_dir.path().to_path_buf(),
-        include_globs: "*.csv".to_string(),
-        exclude_globs: "".to_string(),
+        include_globs: Some("*.csv"),
+        exclude_globs: Some(""),
         include_hidden: false,
         fixed_strings: true,
         match_case: true,
@@ -274,10 +257,7 @@ async fn test_headless_advanced_regex_features() -> anyhow::Result<()> {
     };
 
     let result = run_headless(search_config);
-    assert_eq!(
-        result.unwrap(),
-        "Successful replacements (lines): 4".to_string(),
-    );
+    assert_eq!(result.unwrap(), "Success: 1 file updated".to_string(),);
 
     assert_test_files!(
         &temp_dir,
@@ -331,11 +311,11 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         // Include glob - only match Rust files
         let search_config = SearchConfiguration {
-            search_text: "REPLACE_ME".to_string(),
-            replacement_text: "REPLACED_CODE".to_string(),
+            search_text: "REPLACE_ME",
+            replacement_text: "REPLACED_CODE",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "**/*.rs".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some("**/*.rs"),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -345,10 +325,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 4".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 4 files updated".to_string(),);
 
         assert_test_files!(
             &temp_dir,
@@ -376,11 +353,11 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         // Exclude glob - exclude test files
         let search_config = SearchConfiguration {
-            search_text: "REPLACED_CODE".to_string(),
-            replacement_text: "FINAL_VERSION".to_string(),
+            search_text: "REPLACED_CODE",
+            replacement_text: "FINAL_VERSION",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "**/*.rs".to_string(),
-            exclude_globs: "tests/**".to_string(),
+            include_globs: Some("**/*.rs"),
+            exclude_globs: Some("tests/**"),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -390,10 +367,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 2".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 2 files updated".to_string(),);
 
         assert_test_files!(
             &temp_dir,
@@ -421,11 +395,11 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         // Multiple include globs
         let search_config = SearchConfiguration {
-            search_text: "REPLACE_ME".to_string(),
-            replacement_text: "DOCS_REPLACED".to_string(),
+            search_text: "REPLACE_ME",
+            replacement_text: "DOCS_REPLACED",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "**/*.md,**/*.txt".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some("**/*.md,**/*.txt"),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -435,10 +409,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 2".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 2 files updated".to_string(),);
 
         assert_test_files!(
             &temp_dir,
@@ -484,11 +455,11 @@ test_with_both_regex_modes_and_fixed_strings!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: "whole_word".to_string(),
-            replacement_text: "REPLACED".to_string(),
+            search_text: "whole_word",
+            replacement_text: "REPLACED",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -498,10 +469,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 2".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 2 files updated".to_string(),);
 
         assert_test_files!(
             &temp_dir,
@@ -537,11 +505,11 @@ test_with_both_regex_modes_and_fixed_strings!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: "pattern".to_string(),
-            replacement_text: "REPLACED".to_string(),
+            search_text: "pattern",
+            replacement_text: "REPLACED",
             directory: temp_dir1.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -551,10 +519,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 2".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 2 files updated".to_string(),);
 
         assert_test_files!(
             &temp_dir1,
@@ -583,11 +548,11 @@ test_with_both_regex_modes_and_fixed_strings!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: "pattern".to_string(),
-            replacement_text: "variable".to_string(),
+            search_text: "pattern",
+            replacement_text: "variable",
             directory: temp_dir2.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: false,
@@ -597,10 +562,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 5".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 2 files updated".to_string(),);
 
         assert_test_files!(
             &temp_dir2,
@@ -631,11 +593,11 @@ test_with_both_regex_modes_and_fixed_strings!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: "PATTERN".to_string(),
-            replacement_text: "REPLACEMENT".to_string(),
+            search_text: "PATTERN",
+            replacement_text: "REPLACEMENT",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -645,10 +607,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 1".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 1 file updated".to_string(),);
 
         assert_test_files!(
             &temp_dir,
@@ -682,11 +641,11 @@ test_with_both_regex_modes_and_fixed_strings!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: "PATTERN".to_string(),
-            replacement_text: "REPLACEMENT".to_string(),
+            search_text: "PATTERN",
+            replacement_text: "REPLACEMENT",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -696,10 +655,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 4".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 2 files updated".to_string(),);
 
         assert_test_files!(
             &temp_dir,
@@ -727,11 +683,11 @@ test_with_both_regex_modes_and_fixed_strings!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: "PATTERN".to_string(),
-            replacement_text: "REPLACEMENT".to_string(),
+            search_text: "PATTERN",
+            replacement_text: "REPLACEMENT",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -741,10 +697,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 2".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 2 files updated".to_string(),);
 
         assert_test_files!(
             &temp_dir,
@@ -777,11 +730,11 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         // Default behavior - hidden files excluded
         let search_config = SearchConfiguration {
-            search_text: "PATTERN".to_string(),
-            replacement_text: "REPLACEMENT".to_string(),
+            search_text: "PATTERN",
+            replacement_text: "REPLACEMENT",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false, // Default behavior
             fixed_strings,
             match_case: true,
@@ -791,10 +744,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 1".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 1 file updated".to_string(),);
 
         // Only visible file should be modified, hidden files untouched
         assert_test_files!(
@@ -812,11 +762,11 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         // Explicit include hidden files
         let search_config = SearchConfiguration {
-            search_text: "PATTERN".to_string(),
-            replacement_text: "REPLACEMENT".to_string(),
+            search_text: "PATTERN",
+            replacement_text: "REPLACEMENT",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: true, // Include hidden files
             fixed_strings,
             match_case: true,
@@ -826,10 +776,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 2".to_string(),
-        );
+        assert_eq!(result.unwrap(), "Success: 2 files updated".to_string(),);
 
         // Now all files should be modified
         assert_test_files!(
@@ -859,11 +806,11 @@ test_with_both_regex_modes!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: "(".to_string(), // Unclosed parenthesis = invalid regex
-            replacement_text: "replacement".to_string(),
+            search_text: "(", // Unclosed parenthesis = invalid regex
+            replacement_text: "replacement",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings: false,
             match_case: true,
@@ -897,11 +844,11 @@ test_with_both_regex_modes_and_fixed_strings!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: "valid".to_string(),
-            replacement_text: "replacement".to_string(),
+            search_text: "valid",
+            replacement_text: "replacement",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "{{".to_string(), // Invalid glob pattern
-            exclude_globs: "".to_string(),
+            include_globs: Some("{{"), // Invalid glob pattern
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -967,11 +914,11 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         // Test include glob - only include .txt files
         let search_config = SearchConfiguration {
-            search_text: "PATTERN".to_string(),
-            replacement_text: "REPLACEMENT".to_string(),
+            search_text: "PATTERN",
+            replacement_text: "REPLACEMENT",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "*.txt".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some("*.txt"),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -981,10 +928,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 2".to_owned(),
-        );
+        assert_eq!(result.unwrap(), "Success: 2 files updated".to_owned(),);
 
         // Verify only .txt files were modified
         assert_test_files!(
@@ -1026,11 +970,11 @@ test_with_both_regex_modes_and_fixed_strings!(
         );
 
         let result = run_headless(SearchConfiguration {
-            search_text: "PATTERN".to_string(),
-            replacement_text: "REPLACEMENT".to_string(),
+            search_text: "PATTERN",
+            replacement_text: "REPLACEMENT",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "*.txt".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some("*.txt"),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -1038,10 +982,7 @@ test_with_both_regex_modes_and_fixed_strings!(
             advanced_regex,
         });
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 0".to_owned(),
-        );
+        assert_eq!(result.unwrap(), "Success: 0 files updated".to_owned(),);
 
         Ok(())
     }
@@ -1089,11 +1030,11 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         // Test exclude glob - exclude .txt files
         let search_config = SearchConfiguration {
-            search_text: "PATTERN".to_string(),
-            replacement_text: "REPLACEMENT".to_string(),
+            search_text: "PATTERN",
+            replacement_text: "REPLACEMENT",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "*.txt".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some("*.txt"),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -1103,10 +1044,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 2".to_owned(),
-        );
+        assert_eq!(result.unwrap(), "Success: 2 files updated".to_owned(),);
 
         // Verify non-.txt files were modified
         assert_test_files!(
@@ -1236,11 +1174,11 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         // Include all .rs files but exclude those in tests directory
         let search_config = SearchConfiguration {
-            search_text: "PATTERN".to_string(),
-            replacement_text: "REPLACEMENT".to_string(),
+            search_text: "PATTERN",
+            replacement_text: "REPLACEMENT",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "**/*.rs".to_string(),
-            exclude_globs: "tests/**".to_string(),
+            include_globs: Some("**/*.rs"),
+            exclude_globs: Some("tests/**"),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -1250,10 +1188,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 6".to_owned(),
-        );
+        assert_eq!(result.unwrap(), "Success: 3 files updated".to_owned(),);
 
         // Verify only source .rs files were modified, not test files or docs
         assert_test_files!(
@@ -1373,11 +1308,11 @@ test_with_both_regex_modes_and_fixed_strings!(
         );
 
         let search_config = SearchConfiguration {
-            search_text: "PATTERN".to_string(),
-            replacement_text: "REPLACED".to_string(),
+            search_text: "PATTERN",
+            replacement_text: "REPLACED",
             directory: temp_dir.path().to_path_buf(),
-            include_globs: "".to_string(),
-            exclude_globs: "".to_string(),
+            include_globs: Some(""),
+            exclude_globs: Some(""),
             include_hidden: false,
             fixed_strings,
             match_case: true,
@@ -1387,10 +1322,7 @@ test_with_both_regex_modes_and_fixed_strings!(
 
         let result = run_headless(search_config);
         assert!(result.is_ok());
-        assert_eq!(
-            result.unwrap(),
-            "Successful replacements (lines): 4".to_owned(),
-        );
+        assert_eq!(result.unwrap(), "Success: 2 files updated".to_owned(),);
 
         assert_test_files!(
             &temp_dir,
